#include	<htc.h>						//generic include for all devices
#include "Onewire.h"


/*************************************************************************************************************************************************
		OnewireReset(): All transactions on the 1-Wire bus begin with an initialization sequence. The initialization sequence consists of a reset
						pulse transmitted by the bus master followed by presence pulse(s) transmitted by the slave(s).
*************************************************************************************************************************************************/
char OnewireReset( void )
{
	OW_LO
	__delay_us( 480 );							// 480us minimum
	OW_RLS
  	__delay_us( 40 ); 							// slave waits 15-60us
  	if( OWPORTIN & OWPORTPIN ) 					// line should be pulled down by slave
		return 1;
  	__delay_us( 300 );							// slave TX presence pulse 60-240us
	if( !(OWPORTIN & OWPORTPIN) )				// line should be "released" by slave
		return 2;
  return 0;
}



/*************************************************************************************************************************************************
		OnewireWriteBit(): Follows the onewire protocol to write a single bit to a device.
*************************************************************************************************************************************************/
void OnewireWriteBit( char cBit )
{
	__delay_us( 1 );							// recovery, min 1us
	OW_HI
	if( cBit )
	{
		OW_LO
    	__delay_us( 5 );						// max 15us
		OW_RLS									// input
    	__delay_us( 56 );
  	}
	else
	{
	  	OW_LO
	  	__delay_us( 60 );						// min 60us
	  	OW_RLS									// input
	  	__delay_us( 1 );
  	}
 }


/*************************************************************************************************************************************************
		OnewireReadBit(): Follows the onewire protocol to read a single bit from a device.
*************************************************************************************************************************************************/
char OnewireReadBit( void )
{
	char cBit = 0;
 	__delay_us( 1 );							// recovery, min 1us
  	OW_LO
  	__delay_us( 5 );							// hold min 1us
  	OW_RLS
  	__delay_us( 5 );							// 15us window
  	if( OWPORTIN & OWPORTPIN )
		cBit = 1;
  	__delay_us( 46 );							// rest of the read slot
  	return cBit;
}


/*************************************************************************************************************************************************
		OnewireWriteByte(): Follows the onewire protocol to write a single byte to a device.
*************************************************************************************************************************************************/
void OnewireWriteByte( unsigned char ucByte )
{
	char cLoop;
	for( cLoop = 0; cLoop < 8; cLoop++)
	{
    	OnewireWriteBit( ucByte & 1 );
    	ucByte >>= 1;
  	}
}


/*************************************************************************************************************************************************
		OnewireReadByte(): Follows the onewire protocol to read a single byte from a device.
*************************************************************************************************************************************************/
unsigned char OnewireReadByte( void )
{
	char cLoop;
  	unsigned char ucByte = 0;
  	for( cLoop = 0; cLoop < 8; cLoop++)
  	{
    	ucByte >>= 1;
    	if( OnewireReadBit() )
			ucByte |= 0x80;
  	}
  	return ucByte;
}


/*************************************************************************************************************************************************
		ReadDS1820(): Follows the onewire protocol to read 16 bits from the DS1820.
*************************************************************************************************************************************************/
unsigned int ReadDS1820( void )
{
	unsigned char cLoop;
	unsigned int uiValue = 0;
	for( cLoop = 0; cLoop < 16; cLoop++)
	{
		uiValue >>= 1;
		if( OnewireReadBit() )
			uiValue |= 0x8000;
	}
  	return uiValue;
}


/*************************************************************************************************************************************************
		GetData(): Follows the onewire protocol to read the temperature for the DS1820.
*************************************************************************************************************************************************/
unsigned int GetData( void )
{
    unsigned int uiTemp;
	OnewireReset();
	OnewireWriteByte( 0xCC );							// skip ROM command
	OnewireWriteByte( 0x44 );							// convert T command
	OW_HI
	__delay_ms( 850 ); 									// at least 750 ms for the default 12-bit resolution
	OnewireReset();
	OnewireWriteByte( 0xCC );							// skip ROM command
	OnewireWriteByte( 0xBE );							// read scratchpad command
	uiTemp = ReadDS1820();
    if( uiTemp < 0x8000)								//Is the temperature above zero?
//	    return( uiTemp >> 1 );							//If Yes, convert this way.
	    return uiTemp;									//If Yes, convert this way.
    else												//If No,
    {
        uiTemp = ( ~uiTemp ) + 1;						//		form the 2's complement
//        return( uiTemp >> 1 );						//								and convert as above.
        return uiTemp;									//								and convert as above.
    }
}
